@@ -18,45 +18,48 @@
 */
 
 #include <stdlib.h>
-
-#include <gtk/gtk.h>
-#include <gdk/gdkx.h>
-
 #include <math.h>
+
 #include <cairo.h>
+#include <gtk/gtk.h>
+#include <GL/gl.h>
+#if GTK_CHECK_VERSION (3, 10, 0)
+#include "gtk3imagemenuitem.h"
+#endif
 
-#include <GL/gl.h> 
-
-#include "cairo-dock-notifications.h"
 #include "cairo-dock-icon-facility.h"  // cairo_dock_compute_icon_area
 #include "cairo-dock-dock-facility.h"  // cairo_dock_is_hidden
-#include "cairo-dock-module-factory.h"  // cairo_dock_search_container_from_icon
-#include "cairo-dock-dock-manager.h"  // cairo_dock_search_dock_from_name
-#include "cairo-dock-desklet-factory.h"  // cairo_dock_search_container_from_icon
+#include "cairo-dock-dock-manager.h"  // gldi_dock_get
 #include "cairo-dock-dialog-manager.h"
 #include "cairo-dock-log.h"
 #include "cairo-dock-config.h"
+#include "cairo-dock-utils.h"  // cairo_dock_string_is_address
+#include "cairo-dock-windows-manager.h"  // gldi_windows_get_active
 #include "cairo-dock-opengl.h"
-#include "cairo-dock-notifications.h"
 #include "cairo-dock-animations.h"  // cairo_dock_animation_will_be_visible
-#include "cairo-dock-X-manager.h"  // gldi_get_desktop_width
+#include "cairo-dock-desktop-manager.h"  // gldi_desktop_get_width
+#include "cairo-dock-menu.h"  // gldi_menu_new
 #define _MANAGER_DEF_
 #include "cairo-dock-container.h"
 
 // public (manager, config, data)
-CairoContainersParam myContainersParam;
-CairoContainersManager myContainersMgr;
-CairoContainer *g_pPrimaryContainer = NULL;
-CairoDockDesktopBackground *g_pFakeTransparencyDesktopBg = NULL;
+GldiContainersParam myContainersParam;
+GldiManager myContainersMgr;
+GldiObjectManager myContainerObjectMgr;
+GldiContainer *g_pPrimaryContainer = NULL;
+GldiDesktopBackground *g_pFakeTransparencyDesktopBg = NULL;
 
 // dependancies
 extern CairoDockGLConfig g_openglConfig;
 extern gboolean g_bUseOpenGL;
 extern CairoDockHidingEffect *g_pHidingBackend;  // cairo_dock_is_hidden
+extern CairoDock *g_pMainDock;  // for the default dock visibility when composite goes off->on
 
 // private
 static gboolean s_bSticky = TRUE;
 static gboolean s_bInitialOpacity0 = TRUE;  // set initial window opacity to 0, to avoid grey rectangles.
+static gboolean s_bNoComposite = FALSE;
+static GldiContainerManagerBackend s_backend;
 
 
 void cairo_dock_set_containers_non_sticky (void)
@@ -86,7 +89,7 @@ static gboolean _prevent_delete (G_GNUC_UNUSED GtkWidget *pWidget, G_GNUC_UNUSED
 	return TRUE;  // on empeche les ALT+F4 malheureux.
 }
 
-static void cairo_dock_set_default_rgba_visual (GtkWidget *pWidget)
+void cairo_dock_set_default_rgba_visual (GtkWidget *pWidget)
 {
 	GdkScreen* pScreen = gtk_widget_get_screen (pWidget);
 	
@@ -105,7 +108,7 @@ static void cairo_dock_set_default_rgba_visual (GtkWidget *pWidget)
 	#endif
 }
 
-static gboolean _cairo_default_container_animation_loop (CairoContainer *pContainer)
+static gboolean _cairo_default_container_animation_loop (GldiContainer *pContainer)
 {
 	gboolean bContinue = FALSE;
 	
@@ -120,10 +123,10 @@ static gboolean _cairo_default_container_animation_loop (CairoContainer *pContai
 	
 	if (bUpdateSlowAnimation)
 	{
-		cairo_dock_notify_on_object (pContainer, NOTIFICATION_UPDATE_SLOW, pContainer, &pContainer->bKeepSlowAnimation);
+		gldi_object_notify (pContainer, NOTIFICATION_UPDATE_SLOW, pContainer, &pContainer->bKeepSlowAnimation);
 	}
 	
-	cairo_dock_notify_on_object (pContainer, NOTIFICATION_UPDATE, pContainer, &bContinue);
+	gldi_object_notify (pContainer, NOTIFICATION_UPDATE, pContainer, &bContinue);
 	
 	if (! bContinue && ! pContainer->bKeepSlowAnimation)
 	{
@@ -140,101 +143,31 @@ static gboolean _set_opacity (GtkWidget *pWidget,
 #else
 	G_GNUC_UNUSED cairo_t *ctx,
 #endif
-	CairoContainer *pContainer)
+	GldiContainer *pContainer)
 {
 	if (pContainer->iWidth != 1 ||pContainer->iHeight != 1)
 	{
 		g_signal_handlers_disconnect_by_func (pWidget, _set_opacity, pContainer);  // we'll never need to pass here any more, so simply disconnect ourselves.
 		//g_print ("____OPACITY 1 (%dx%d)\n", pContainer->iWidth, pContainer->iHeight);
+		#if GTK_CHECK_VERSION (3, 8, 0)
+		gtk_widget_set_opacity (pWidget, 1.);
+		#else
 		gtk_window_set_opacity (GTK_WINDOW (pWidget), 1.);
+		#endif
 	}
 	return FALSE ;
 }
-GtkWidget *cairo_dock_init_container_full (CairoContainer *pContainer, gboolean bOpenGLWindow)
+
+static void _remove_background (G_GNUC_UNUSED GtkWidget *pWidget, GldiContainer *pContainer)
 {
-	GtkWidget* pWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	pContainer->pWidget = pWindow;
-	
-	if (!pContainer->iface.animation_loop)
-		pContainer->iface.animation_loop = _cairo_default_container_animation_loop;
-	
-	if (g_bUseOpenGL && bOpenGLWindow)
-	{
-		gldi_glx_init_container (pContainer);
-		pContainer->iAnimationDeltaT = myContainersParam.iGLAnimationDeltaT;
-	}
-	else
-	{
-		cairo_dock_set_default_rgba_visual (pWindow);
-		pContainer->iAnimationDeltaT = myContainersParam.iCairoAnimationDeltaT;
-	}
-	if (pContainer->iAnimationDeltaT == 0)
-		pContainer->iAnimationDeltaT = 30;
-	pContainer->fRatio = 1;
-	
-	g_signal_connect (G_OBJECT (pWindow),
-		"delete-event",
-		G_CALLBACK (_prevent_delete),
-		NULL);
-	
-	gtk_window_set_default_size (GTK_WINDOW (pWindow), 1, 1);  // this should prevent having grey rectangles during the loading, when the window is mapped and rendered by the WM but not yet by us.
-	gtk_window_resize (GTK_WINDOW (pWindow), 1, 1);
-	gtk_widget_set_app_paintable (pWindow, TRUE);
-	gtk_window_set_decorated (GTK_WINDOW (pWindow), FALSE);
-	gtk_window_set_skip_pager_hint (GTK_WINDOW(pWindow), TRUE);
-	gtk_window_set_skip_taskbar_hint (GTK_WINDOW(pWindow), TRUE);
-	if (s_bSticky)
-		gtk_window_stick (GTK_WINDOW (pWindow));
-	// set the opacity to 0 to avoid seeing grey rectangles until the window is ready to be painted by us.
-	if (s_bInitialOpacity0)
-	{
-		gtk_window_set_opacity (GTK_WINDOW (pWindow), 0.);
-		g_signal_connect (G_OBJECT (pWindow),
-			#if (GTK_MAJOR_VERSION < 3)
-			"expose-event",
-			#else
-			"draw",
-			#endif
-			G_CALLBACK (_set_opacity),
-			pContainer);  // the callback will be removed once it has done its job.
-	}
-	
-	// needed since gtk+-3.0 but it's possible that this resize grip has been backported to gtk+-2.0 (e.g. in Ubuntu Natty...)
-	#if (GTK_MAJOR_VERSION >= 3 || ENABLE_GTK_GRIP == 1)
-	gtk_window_set_has_resize_grip (GTK_WINDOW(pWindow), FALSE);
+	#if (GTK_MAJOR_VERSION < 3)
+	gdk_window_set_back_pixmap (gldi_container_get_gdk_window (pContainer), NULL, FALSE);
+	#else
+	gdk_window_set_background_pattern (gldi_container_get_gdk_window (pContainer), NULL);  // window must be realized (shown)
 	#endif
-	
-	if (GLDI_OBJECT (pContainer)->mgr == NULL)
-		gldi_object_set_manager (GLDI_OBJECT (pContainer), GLDI_MANAGER (&myContainersMgr));  // the implementation of the container will set its manager on top of this one.
-	
-	if (g_pPrimaryContainer == NULL)
-	{
-		g_pPrimaryContainer = pContainer;
-	}
-	return pWindow;
 }
 
-
-void cairo_dock_finish_container (CairoContainer *pContainer)
-{
-	gldi_glx_finish_container (pContainer);
-	
-	gtk_widget_destroy (pContainer->pWidget);  // enleve les signaux.
-	pContainer->pWidget = NULL;
-	if (pContainer->iSidGLAnimation != 0)
-	{
-		g_source_remove (pContainer->iSidGLAnimation);
-		pContainer->iSidGLAnimation = 0;
-	}
-	cairo_dock_notify_on_object (pContainer, NOTIFICATION_DESTROY, pContainer);
-	cairo_dock_clear_notifications_on_object (pContainer);
-	
-	if (g_pPrimaryContainer == pContainer)
-		g_pPrimaryContainer = NULL;
-}
-
-
-void cairo_dock_redraw_container (CairoContainer *pContainer)
+void cairo_dock_redraw_container (GldiContainer *pContainer)
 {
 	g_return_if_fail (pContainer != NULL);
 	GdkRectangle rect = {0, 0, pContainer->iWidth, pContainer->iHeight};
@@ -246,7 +179,7 @@ void cairo_dock_redraw_container (CairoContainer *pContainer)
 	cairo_dock_redraw_container_area (pContainer, &rect);
 }
 
-static inline void _redraw_container_area (CairoContainer *pContainer, GdkRectangle *pArea)
+static inline void _redraw_container_area (GldiContainer *pContainer, GdkRectangle *pArea)
 {
 	g_return_if_fail (pContainer != NULL);
 	if (! gldi_container_is_visible (pContainer))
@@ -263,16 +196,18 @@ static inline void _redraw_container_area (CairoContainer *pContainer, GdkRectan
 		gdk_window_invalidate_rect (gldi_container_get_gdk_window (pContainer), pArea, FALSE);
 }
 
-void cairo_dock_redraw_container_area (CairoContainer *pContainer, GdkRectangle *pArea)
+void cairo_dock_redraw_container_area (GldiContainer *pContainer, GdkRectangle *pArea)
 {
 	if (CAIRO_DOCK_IS_DOCK (pContainer) && ! cairo_dock_animation_will_be_visible (CAIRO_DOCK (pContainer)))  // inutile de redessiner.
 		return ;
 	_redraw_container_area (pContainer, pArea);
 }
 
-void cairo_dock_redraw_icon (Icon *icon, CairoContainer *pContainer)
+void cairo_dock_redraw_icon (Icon *icon)
 {
-	g_return_if_fail (icon != NULL && pContainer != NULL);
+	g_return_if_fail (icon != NULL);
+	GldiContainer *pContainer = cairo_dock_get_icon_container (icon);
+	g_return_if_fail (pContainer != NULL);
 	GdkRectangle rect;
 	cairo_dock_compute_icon_area (icon, pContainer, &rect);
 	
@@ -284,40 +219,6 @@ void cairo_dock_redraw_icon (Icon *icon, CairoContainer *pContainer)
 }
 
 
-static gboolean _cairo_dock_search_icon_in_desklet (CairoDesklet *pDesklet, Icon *icon)
-{
-	if (pDesklet->icons != NULL)
-	{
-		Icon *pIcon;
-		GList *ic;
-		for (ic = pDesklet->icons; ic != NULL; ic = ic->next)
-		{
-			pIcon = ic->data;
-			if (pIcon == icon)
-				return TRUE;
-		}
-	}
-	return FALSE;
-}
-CairoContainer *cairo_dock_search_container_from_icon (Icon *icon)
-{
-	g_return_val_if_fail (icon != NULL, NULL);
-	if (CAIRO_DOCK_IS_APPLET (icon))
-	{
-		return icon->pModuleInstance->pContainer;
-	}
-	else if (icon->cParentDockName != NULL)
-	{
-		return CAIRO_CONTAINER (cairo_dock_search_dock_from_name (icon->cParentDockName));
-	}
-	else
-	{
-		CairoDesklet *pDesklet = cairo_dock_foreach_desklet ((CairoDockForeachDeskletFunc)_cairo_dock_search_icon_in_desklet, icon);
-		return CAIRO_CONTAINER (pDesklet);
-	}
-}
-
-
 void cairo_dock_allow_widget_to_receive_data (GtkWidget *pWidget, GCallback pCallBack, gpointer data)
 {
 	// /*GtkTargetEntry pTargetEntry[6] = {0};
@@ -348,34 +249,12 @@ void cairo_dock_allow_widget_to_receive_data (GtkWidget *pWidget, GCallback pCal
 		data);
 }
 
-void gldi_container_disable_drop (CairoContainer *pContainer)
+void gldi_container_disable_drop (GldiContainer *pContainer)
 {
 	gtk_drag_dest_set_target_list (pContainer->pWidget, NULL);
 }
 
-gboolean cairo_dock_string_is_adress (const gchar *cString)
-{
-	gchar *protocole = g_strstr_len (cString, -1, "://");
-	if (protocole == NULL || protocole == cString)
-	{
-		if (strncmp (cString, "www", 3) == 0)
-			return TRUE;
-		return FALSE;
-	}
-	const gchar *str = cString;
-	while (*str == ' ')
-		str ++;
-	while (str < protocole)
-	{
-		if (! g_ascii_isalnum (*str) && *str != '-')  // x-nautilus-desktop://
-			return FALSE;
-		str ++;
-	}
-	
-	return TRUE;
-}
-
-void cairo_dock_notify_drop_data (gchar *cReceivedData, Icon *pPointedIcon, double fOrder, CairoContainer *pContainer)
+void gldi_container_notify_drop_data (GldiContainer *pContainer, gchar *cReceivedData, Icon *pPointedIcon, double fOrder)
 {
 	g_return_if_fail (cReceivedData != NULL);
 	gchar *cData = NULL;
@@ -387,12 +266,12 @@ void cairo_dock_notify_drop_data (gchar *cReceivedData, Icon *pPointedIcon, doub
 	{
 		g_string_assign (sArg, cStringList[i]);
 		
-		if (! cairo_dock_string_is_adress (cStringList[i]))
+		if (! cairo_dock_string_is_address (cStringList[i]))
 		{
 			j = i + 1;
 			while (cStringList[j] != NULL)
 			{
-				if (cairo_dock_string_is_adress (cStringList[j]))
+				if (cairo_dock_string_is_address (cStringList[j]))
 					break ;
 				g_string_append_printf (sArg, "\n%s", cStringList[j]);
 				j ++;
@@ -412,7 +291,7 @@ void cairo_dock_notify_drop_data (gchar *cReceivedData, Icon *pPointedIcon, doub
 		
 		cData = sArg->str;
 		cd_debug (" notification de drop '%s'", cData);
-		cairo_dock_notify_on_object (pContainer, NOTIFICATION_DROP_DATA, cData, pPointedIcon, fOrder, pContainer);
+		gldi_object_notify (pContainer, NOTIFICATION_DROP_DATA, cData, pPointedIcon, fOrder, pContainer);
 	}
 	
 	g_strfreev (cStringList);
@@ -420,144 +299,73 @@ void cairo_dock_notify_drop_data (gchar *cReceivedData, Icon *pPointedIcon, doub
 }
 
 
-gboolean cairo_dock_emit_signal_on_container (CairoContainer *pContainer, const gchar *cSignal)
+void gldi_container_reserve_space (GldiContainer *pContainer, int left, int right, int top, int bottom, int left_start_y, int left_end_y, int right_start_y, int right_end_y, int top_start_x, int top_end_x, int bottom_start_x, int bottom_end_x)
+{
+	if (s_backend.reserve_space)
+		s_backend.reserve_space (pContainer, left, right, top, bottom, left_start_y, left_end_y, right_start_y, right_end_y, top_start_x, top_end_x, bottom_start_x, bottom_end_x);
+}
+
+int gldi_container_get_current_desktop_index (GldiContainer *pContainer)
+{
+	if (s_backend.get_current_desktop_index)
+		return s_backend.get_current_desktop_index (pContainer);
+	return 0;
+}
+
+void gldi_container_move (GldiContainer *pContainer, int iNumDesktop, int iAbsolutePositionX, int iAbsolutePositionY)
+{
+	if (s_backend.move)
+		s_backend.move (pContainer, iNumDesktop, iAbsolutePositionX, iAbsolutePositionY);
+}
+
+gboolean gldi_container_is_active (GldiContainer *pContainer)
+{
+	if (s_backend.is_active)
+		return s_backend.is_active (pContainer);
+	return FALSE;
+}
+
+void gldi_container_present (GldiContainer *pContainer)
+{
+	if (s_backend.present)
+		s_backend.present (pContainer);
+}
+
+void gldi_container_manager_register_backend (GldiContainerManagerBackend *pBackend)
+{
+	gpointer *ptr = (gpointer*)&s_backend;
+	gpointer *src = (gpointer*)pBackend;
+	gpointer *src_end = (gpointer*)(pBackend + 1);
+	while (src != src_end)
+	{
+		if (*src != NULL)
+			*ptr = *src;
+		src ++;
+		ptr ++;
+	}
+}
+
+
+gboolean cairo_dock_emit_signal_on_container (GldiContainer *pContainer, const gchar *cSignal)
 {
 	static gboolean bReturn;
 	g_signal_emit_by_name (pContainer->pWidget, cSignal, NULL, &bReturn);
 	return FALSE;
 }
-gboolean cairo_dock_emit_leave_signal (CairoContainer *pContainer)
+gboolean cairo_dock_emit_leave_signal (GldiContainer *pContainer)
 {
 	// actualize the coordinates of the pointer, since they are most probably out-dated (because the mouse has left the dock, or because a right-click generates an event with (0;0) coordinates)
 	gldi_container_update_mouse_position (pContainer);
 	return cairo_dock_emit_signal_on_container (pContainer, "leave-notify-event");
 }
-gboolean cairo_dock_emit_enter_signal (CairoContainer *pContainer)
+gboolean cairo_dock_emit_enter_signal (GldiContainer *pContainer)
 {
 	return cairo_dock_emit_signal_on_container (pContainer, "enter-notify-event");
 }
 
 
-static void _place_menu_on_icon (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer *data)
-{
-	*push_in = FALSE;
-	Icon *pIcon = data[0];
-	CairoContainer *pContainer = data[1];
-	int x0 = pContainer->iWindowPositionX + pIcon->fDrawX;
-	int y0 = pContainer->iWindowPositionY + pIcon->fDrawY;
-	
-	int w, h;  // taille menu
-	GtkRequisition requisition;
-	#if (GTK_MAJOR_VERSION < 3)
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
-	#else
-	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
-	#endif
-	w = requisition.width;
-	h = requisition.height;
-	
-	int Hs = (pContainer->bIsHorizontal ? gldi_get_desktop_height() : gldi_get_desktop_width());
-	//g_print ("%d;%d %dx%d\n", x0, y0, w, h);
-	if (pContainer->bIsHorizontal)
-	{
-		*x = x0;
-		if (y0 > Hs/2)  // pContainer->bDirectionUp
-			*y = y0 - h;
-		else
-			*y = y0 + pIcon->fHeight * pIcon->fScale;
-	}
-	else
-	{
-		*y = MIN (x0, gldi_get_desktop_height() - h);
-		if (y0 > Hs/2)  // pContainer->bDirectionUp
-			*x = y0 - w;
-		else
-			*x = y0 + pIcon->fHeight * pIcon->fScale;
-	}
-}
-void cairo_dock_popup_menu_on_icon (GtkWidget *menu, Icon *pIcon, CairoContainer *pContainer)
-{
-	static gpointer data[2];  // 1 seul menu a la fois, donc on peut la faire statique.
-	
-	if (menu == NULL)
-		return;
-	GtkMenuPositionFunc place_menu = NULL;  // if 'place_menu' is NULL, then 'data' is ignored.
-	if (pIcon != NULL && pContainer != NULL)
-	{
-		place_menu = (GtkMenuPositionFunc)_place_menu_on_icon;
-		data[0] = pIcon;
-		data[1] = pContainer;
-	}
-	
-	if (pContainer->iface.setup_menu)
-		pContainer->iface.setup_menu (pContainer, pIcon, menu);
-	
-	gtk_widget_show_all (GTK_WIDGET (menu));
-	
-	gtk_menu_popup (GTK_MENU (menu),
-		NULL,
-		NULL,
-		place_menu,
-		data,
-		1,
-		gtk_get_current_event_time ());
-}
-
-
-GtkWidget *cairo_dock_add_in_menu_with_stock_and_data (const gchar *cLabel, const gchar *gtkStock, GCallback pFunction, GtkWidget *pMenu, gpointer pData)
-{
-	GtkWidget *pMenuItem = gtk_image_menu_item_new_with_label (cLabel);
-	if (gtkStock)
-	{
-		GtkWidget *image = NULL;
-		if (*gtkStock == '/')
-		{
-			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (gtkStock, 16, 16, NULL);
-			image = gtk_image_new_from_pixbuf (pixbuf);
-			g_object_unref (pixbuf);
-		}
-		else
-		{
-			image = gtk_image_new_from_stock (gtkStock, GTK_ICON_SIZE_MENU);
-		}
-		_gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (pMenuItem), image);
-	}
-	gtk_menu_shell_append  (GTK_MENU_SHELL (pMenu), pMenuItem);
-	if (pFunction)
-		g_signal_connect (G_OBJECT (pMenuItem), "activate", G_CALLBACK (pFunction), pData);
-	return pMenuItem;
-}
-
-GtkWidget *cairo_dock_create_sub_menu (const gchar *cLabel, GtkWidget *pMenu, const gchar *cImage)
-{
-	GtkWidget *pMenuItem, *image, *pSubMenu = gtk_menu_new ();
-	if (cImage == NULL)
-	{
-		pMenuItem = gtk_image_menu_item_new_with_label (cLabel);
-	}
-	else
-	{
-		pMenuItem = gtk_image_menu_item_new_with_label (cLabel);
-		if (*cImage == '/')
-		{
-			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (cImage, 24, 24, NULL);
-			image = gtk_image_new_from_pixbuf (pixbuf);
-			g_object_unref (pixbuf);
-		}
-		else
-		{
-			image = gtk_image_new_from_stock (cImage, GTK_ICON_SIZE_MENU);
-		}
-		_gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (pMenuItem), image);
-	}
-	gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem); 
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (pMenuItem), pSubMenu);
-	return pSubMenu; 
-}
-
-
 static GtkWidget *s_pMenu = NULL;  // right-click menu
-GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer)
+GtkWidget *gldi_container_build_menu (GldiContainer *pContainer, Icon *icon)
 {
 	if (s_pMenu != NULL)
 	{
@@ -567,18 +375,21 @@ GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer)
 	g_return_val_if_fail (pContainer != NULL, NULL);
 	
 	//\_________________________ On construit le menu.
-	GtkWidget *menu = gtk_menu_new ();
+	GtkWidget *menu = gldi_menu_new (icon);
 	
 	//\_________________________ On passe la main a ceux qui veulent y rajouter des choses.
 	gboolean bDiscardMenu = FALSE;
-	cairo_dock_notify_on_object (pContainer, NOTIFICATION_BUILD_CONTAINER_MENU, icon, pContainer, menu, &bDiscardMenu);
+	gldi_object_notify (pContainer, NOTIFICATION_BUILD_CONTAINER_MENU, icon, pContainer, menu, &bDiscardMenu);
 	if (bDiscardMenu)
 	{
 		gtk_widget_destroy (menu);
 		return NULL;
 	}
 	
-	cairo_dock_notify_on_object (pContainer, NOTIFICATION_BUILD_ICON_MENU, icon, pContainer, menu);
+	gldi_object_notify (pContainer, NOTIFICATION_BUILD_ICON_MENU, icon, pContainer, menu);
+	
+	if (pContainer->iface.setup_menu)
+		pContainer->iface.setup_menu (pContainer, icon, menu);
 	
 	s_pMenu = menu;
 	g_object_add_weak_pointer (G_OBJECT (menu), (gpointer*)&s_pMenu);  // will nullify 's_pMenu' as soon as the menu is destroyed.
@@ -586,7 +397,7 @@ GtkWidget *cairo_dock_build_menu (Icon *icon, CairoContainer *pContainer)
 }
 
 
-GldiShape *gldi_container_create_input_shape (CairoContainer *pContainer, int x, int y, int w, int h)
+GldiShape *gldi_container_create_input_shape (GldiContainer *pContainer, int x, int y, int w, int h)
 {
 	if (pContainer->iWidth == 0 || pContainer->iHeight == 0)  // very unlikely to happen, but anyway avoid this case.
 		return NULL;
@@ -630,98 +441,220 @@ GldiShape *gldi_container_create_input_shape (CairoContainer *pContainer, int x,
 }
 
 
+  ////////////
+ /// INIT ///
+////////////
+
+static CairoDockVisibility s_iPrevVisibility = CAIRO_DOCK_NB_VISI;
+static void _set_visibility (CairoDock *pDock, gpointer data)
+{
+	gldi_dock_set_visibility (pDock, GPOINTER_TO_INT (data));
+}
+static void _enable_fake_transparency (void)
+{
+	g_pFakeTransparencyDesktopBg = gldi_desktop_background_get (g_bUseOpenGL);
+	s_bNoComposite = TRUE;
+	s_iPrevVisibility = g_pMainDock->iVisibility;
+	gldi_docks_foreach_root ((GFunc)_set_visibility, GINT_TO_POINTER (CAIRO_DOCK_VISI_KEEP_BELOW));  // set the visibility to 'keep below'; that's the best compromise between accessibility and visual annoyance.
+}
+static void _on_composited_changed (GdkScreen *pScreen, G_GNUC_UNUSED gpointer data)
+{
+	if (!gdk_screen_is_composited (pScreen) || (g_bUseOpenGL && ! g_openglConfig.bAlphaAvailable))
+	{
+		_enable_fake_transparency ();
+	}
+	else  // composite is now ON => disable fake transparency
+	{
+		gldi_desktop_background_destroy (g_pFakeTransparencyDesktopBg);
+		s_bNoComposite = FALSE;
+		g_pFakeTransparencyDesktopBg = NULL;
+		if (s_iPrevVisibility < CAIRO_DOCK_NB_VISI)
+			gldi_docks_foreach_root ((GFunc)_set_visibility, GINT_TO_POINTER (s_iPrevVisibility));  // restore the previous visibility.
+	}
+}
+static gboolean _check_composite_delayed (G_GNUC_UNUSED gpointer data)
+{
+	GdkScreen *pScreen = gdk_screen_get_default ();
+	if (!gdk_screen_is_composited (pScreen) || (g_bUseOpenGL && ! g_openglConfig.bAlphaAvailable))  // no composite available -> load the desktop background
+	{
+		cd_message ("Composite is not available");
+		/**g_pFakeTransparencyDesktopBg = gldi_desktop_background_get (g_bUseOpenGL);  // we don't modify the visibility on startup; if it's the first launch, the user has to notice the problem. and if it's not, just respect his configuration.
+		s_bNoComposite = TRUE;*/
+		_enable_fake_transparency ();  // modify the visibility even on startup, because there is no configuration that is really usable except for 'keep-below'
+	}
+	g_signal_connect (pScreen, "composited-changed", G_CALLBACK (_on_composited_changed), NULL);
+	return FALSE;
+}
+static void init (void)
+{
+	g_timeout_add_seconds (4, _check_composite_delayed, NULL);  // we don't want to be annoyed by the activation of the composite on startup
+}
+
   //////////////////
  /// GET CONFIG ///
 //////////////////
 
-static gboolean get_config (GKeyFile *pKeyFile, CairoContainersParam *pContainersParam)
+static gboolean get_config (GKeyFile *pKeyFile, GldiContainersParam *pContainersParam)
 {
 	gboolean bFlushConfFileNeeded = FALSE;
-	pContainersParam->bUseFakeTransparency = cairo_dock_get_boolean_key_value (pKeyFile, "System", "fake transparency", &bFlushConfFileNeeded, FALSE, NULL, NULL);
-	if (g_bUseOpenGL && ! g_openglConfig.bAlphaAvailable)
-		pContainersParam->bUseFakeTransparency = TRUE;
 	
 	int iRefreshFrequency = cairo_dock_get_integer_key_value (pKeyFile, "System", "opengl anim freq", &bFlushConfFileNeeded, 33, NULL, NULL);
 	pContainersParam->iGLAnimationDeltaT = 1000. / iRefreshFrequency;
+	
 	iRefreshFrequency = cairo_dock_get_integer_key_value (pKeyFile, "System", "cairo anim freq", &bFlushConfFileNeeded, 25, NULL, NULL);
 	pContainersParam->iCairoAnimationDeltaT = 1000. / iRefreshFrequency;
-	//g_print ("iGLAnimationDeltaT: %d\n", pContainersParam->iGLAnimationDeltaT);
 	
 	return bFlushConfFileNeeded;
 }
 
-
   ////////////
  /// LOAD ///
 ////////////
 
 static void load (void)
 {
-	if (myContainersParam.bUseFakeTransparency)
+	if (s_bNoComposite)
 	{
-		g_pFakeTransparencyDesktopBg = cairo_dock_get_desktop_background (g_bUseOpenGL);
+		g_pFakeTransparencyDesktopBg = gldi_desktop_background_get (g_bUseOpenGL);
 	}
 }
 
-
-  //////////////
- /// RELOAD ///
-//////////////
-
-static void _set_below (CairoDock *pDock, gpointer data)
-{
-	gtk_window_set_keep_below (GTK_WINDOW (pDock->container.pWidget), GPOINTER_TO_INT (data));
-}
-static void reload (CairoContainersParam *pPrevContainers, CairoContainersParam *pContainers)
-{
-		//\_______________ Fake Transparency.
-	if (pContainers->bUseFakeTransparency && ! pPrevContainers->bUseFakeTransparency)
-	{
-		cairo_dock_foreach_root_docks ((GFunc)_set_below, GINT_TO_POINTER (TRUE));
-		g_pFakeTransparencyDesktopBg = cairo_dock_get_desktop_background (g_bUseOpenGL);
-	}
-	else if (! pContainers->bUseFakeTransparency && pPrevContainers->bUseFakeTransparency)
-	{
-		cairo_dock_foreach_root_docks ((GFunc)_set_below, GINT_TO_POINTER (FALSE));
-		cairo_dock_destroy_desktop_background (g_pFakeTransparencyDesktopBg);
-		g_pFakeTransparencyDesktopBg = NULL;
-	}
-}
-
-
   //////////////
  /// UNLOAD ///
 //////////////
 
 static void unload (void)
 {
-	g_pFakeTransparencyDesktopBg = NULL;  // no need to unref it, since everything is going to be unloaded, including the desktop_bg.
+	gldi_desktop_background_destroy (g_pFakeTransparencyDesktopBg);  // destroy it, since it will be unloaded anyway by the desktop-manager
+	g_pFakeTransparencyDesktopBg = NULL;
 }
 
-
   ///////////////
  /// MANAGER ///
 ///////////////
 
+static void init_object (GldiObject *obj, gpointer attr)
+{
+	GldiContainer *pContainer = (GldiContainer*)obj;
+	GldiContainerAttr *cattr = (GldiContainerAttr*)attr;
+	
+	pContainer->iface.animation_loop = _cairo_default_container_animation_loop;
+	pContainer->fRatio = 1;
+	pContainer->bIsHorizontal = TRUE;
+	pContainer->bDirectionUp = TRUE;
+	
+	// create a window
+	GtkWidget* pWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	pContainer->pWidget = pWindow;
+	gtk_window_set_default_size (GTK_WINDOW (pWindow), 1, 1);  // this should prevent having grey rectangles during the loading, when the window is mapped and rendered by the WM but not yet by us.
+	gtk_window_resize (GTK_WINDOW (pWindow), 1, 1);
+	gtk_widget_set_app_paintable (pWindow, TRUE);
+	gtk_window_set_decorated (GTK_WINDOW (pWindow), FALSE);
+	gtk_window_set_skip_pager_hint (GTK_WINDOW(pWindow), TRUE);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW(pWindow), TRUE);
+	if (s_bSticky)
+		gtk_window_stick (GTK_WINDOW (pWindow));
+	g_signal_connect (G_OBJECT (pWindow),
+		"delete-event",
+		G_CALLBACK (_prevent_delete),
+		NULL);
+	gtk_window_get_size (GTK_WINDOW (pWindow), &pContainer->iWidth, &pContainer->iHeight);  // it's only the initial size allocated by GTK.
+	
+	// set an RGBA visual for cairo or opengl
+	if (g_bUseOpenGL && ! cattr->bNoOpengl)
+	{
+		gldi_gl_container_init (pContainer);
+		pContainer->iAnimationDeltaT = myContainersParam.iGLAnimationDeltaT;
+	}
+	else
+	{
+		cairo_dock_set_default_rgba_visual (pWindow);
+		pContainer->iAnimationDeltaT = myContainersParam.iCairoAnimationDeltaT;
+	}
+	if (pContainer->iAnimationDeltaT == 0)
+		pContainer->iAnimationDeltaT = 30;
+	
+	// set the opacity to 0 to avoid seeing grey rectangles until the window is ready to be painted by us.
+	if (s_bInitialOpacity0)
+	{
+		#if GTK_CHECK_VERSION (3, 8, 0)
+		gtk_widget_set_opacity (pWindow, 0.);
+		#else
+		gtk_window_set_opacity (GTK_WINDOW (pWindow), 0.);
+		#endif
+		g_signal_connect (G_OBJECT (pWindow),
+			#if (GTK_MAJOR_VERSION < 3)
+			"expose-event",
+			#else
+			"draw",
+			#endif
+			G_CALLBACK (_set_opacity),
+			pContainer);  // the callback will be removed once it has done its job.
+	}
+	g_signal_connect (G_OBJECT (pWindow),
+		"realize",
+		G_CALLBACK (_remove_background),
+		pContainer);
+	
+	// remove the resize grip added by gtk3 (it's also possible that this grip has been backported to gtk+-2.0 (e.g. in Ubuntu Natty...))
+	#if (GTK_MAJOR_VERSION >= 3 || ENABLE_GTK_GRIP == 1)
+	gtk_window_set_has_resize_grip (GTK_WINDOW(pWindow), FALSE);
+	#endif
+	
+	// make it the primary container if it's the first
+	if (g_pPrimaryContainer == NULL)
+		g_pPrimaryContainer = pContainer;
+}
+
+static void reset_object (GldiObject *obj)
+{
+	GldiContainer *pContainer = (GldiContainer*)obj;
+	
+	// destroy the opengl context
+	gldi_gl_container_finish (pContainer);
+	
+	// destroy the window (will remove all signals)
+	gtk_widget_destroy (pContainer->pWidget);
+	pContainer->pWidget = NULL;
+	
+	// stop the animation loop
+	if (pContainer->iSidGLAnimation != 0)
+	{
+		g_source_remove (pContainer->iSidGLAnimation);
+		pContainer->iSidGLAnimation = 0;
+	}
+	
+	if (g_pPrimaryContainer == pContainer)
+		g_pPrimaryContainer = NULL;
+}
+
 void gldi_register_containers_manager (void)
 {
 	// Manager
-	memset (&myContainersMgr, 0, sizeof (CairoContainersManager));
-	myContainersMgr.mgr.cModuleName 	= "Containers";
-	myContainersMgr.mgr.init 		= NULL;
-	myContainersMgr.mgr.load 		= load;
-	myContainersMgr.mgr.unload 		= unload;
-	myContainersMgr.mgr.reload 		= (GldiManagerReloadFunc)reload;
-	myContainersMgr.mgr.get_config 	= (GldiManagerGetConfigFunc)get_config;
-	myContainersMgr.mgr.reset_config = (GldiManagerResetConfigFunc)NULL;
+	memset (&myContainersMgr, 0, sizeof (GldiManager));
+	gldi_object_init (GLDI_OBJECT(&myContainersMgr), &myManagerObjectMgr, NULL);
+	myContainersMgr.cModuleName  = "Containers";
+	// interface
+	myContainersMgr.init         = init;
+	myContainersMgr.load         = load;
+	myContainersMgr.unload       = unload;
+	myContainersMgr.reload       = (GldiManagerReloadFunc)NULL;
+	myContainersMgr.get_config   = (GldiManagerGetConfigFunc)get_config;
+	myContainersMgr.reset_config = (GldiManagerResetConfigFunc)NULL;
 	// Config
-	myContainersMgr.mgr.pConfig = (GldiManagerConfigPtr)&myContainersParam;
-	myContainersMgr.mgr.iSizeOfConfig = sizeof (CairoContainersParam);
+	myContainersMgr.pConfig = (GldiManagerConfigPtr)&myContainersParam;
+	myContainersMgr.iSizeOfConfig = sizeof (GldiContainersParam);
 	// data
-	myContainersMgr.mgr.pData = (GldiManagerDataPtr)NULL;
-	myContainersMgr.mgr.iSizeOfData = 0;
+	myContainersMgr.pData = (GldiManagerDataPtr)NULL;
+	myContainersMgr.iSizeOfData = 0;
+	
+	// Object Manager
+	memset (&myContainerObjectMgr, 0, sizeof (GldiObjectManager));
+	myContainerObjectMgr.cName        = "Container";
+	myContainerObjectMgr.iObjectSize  = sizeof (GldiContainer);
+	// interface
+	myContainerObjectMgr.init_object  = init_object;
+	myContainerObjectMgr.reset_object = reset_object;
 	// signals
-	cairo_dock_install_notifications_on_object (&myContainersMgr, NB_NOTIFICATIONS_CONTAINER);
-	// register
-	gldi_register_manager (GLDI_MANAGER(&myContainersMgr));
+	gldi_object_install_notifications (&myContainerObjectMgr, NB_NOTIFICATIONS_CONTAINER);
 }
