@@ -24,33 +24,39 @@
 #include <gtk/gtk.h>
 
 #include "gldi-config.h"
-#include "cairo-dock-notifications.h"
 #include "cairo-dock-icon-factory.h"
-#include "cairo-dock-module-factory.h"  // cairo_dock_deinstanciate_module
+#include "cairo-dock-module-instance-manager.h"  // gldi_module_instance_reload
+#include "cairo-dock-desklet-manager.h"  // gldi_desklets_foreach_icons
+#include "cairo-dock-utils.h"  // cairo_dock_colors_differ
 #include "cairo-dock-log.h"
 #include "cairo-dock-config.h"
 #include "cairo-dock-class-manager.h"  // cairo_dock_deinhibite_class
 #include "cairo-dock-draw.h"  // cairo_dock_render_icon_notification
 #include "cairo-dock-draw-opengl.h"  // cairo_dock_destroy_icon_fbo
 #include "cairo-dock-container.h"
-#include "cairo-dock-dock-manager.h"  // cairo_dock_foreach_icons_in_docks
+#include "cairo-dock-dock-manager.h"  // gldi_icons_foreach_in_docks
 #include "cairo-dock-dialog-manager.h"  // cairo_dock_remove_dialog_if_any
 #include "cairo-dock-data-renderer.h"  // cairo_dock_remove_data_renderer_on_icon
 #include "cairo-dock-file-manager.h"  // cairo_dock_fm_remove_monitor_full
 #include "cairo-dock-animations.h"  // cairo_dock_animation_will_be_visible
 #include "cairo-dock-dock-facility.h"  // cairo_dock_update_dock_size
-#include "cairo-dock-icon-facility.h"  // cairo_dock_foreach_icons_of_type
+#include "cairo-dock-icon-facility.h"  // gldi_icons_foreach_of_type
 #include "cairo-dock-keyfile-utilities.h"  // cairo_dock_open_key_file
 #include "cairo-dock-indicator-manager.h"  // cairo_dock_unload_indicator_textures
-#include "cairo-dock-X-manager.h"  // cairo_dock_get_current_desktop_and_viewport
-#include "cairo-dock-applications-manager.h"  // cairo_dock_unregister_appli
+#include "cairo-dock-desktop-manager.h"  // gldi_desktop_get_current
+#include "cairo-dock-user-icon-manager.h"  // GLDI_OBJECT_IS_USER_ICON
+#include "cairo-dock-separator-manager.h"  // GLDI_OBJECT_IS_SEPARATOR_ICON
+#include "cairo-dock-applications-manager.h"  // GLDI_OBJECT_IS_APPLI_ICON
+#include "cairo-dock-launcher-manager.h"  // GLDI_OBJECT_IS_LAUNCHER_ICON
+#include "cairo-dock-applet-manager.h"  // GLDI_OBJECT_IS_APPLET_ICON
 #include "cairo-dock-backends-manager.h"  // cairo_dock_foreach_icon_container_renderer
 #define _MANAGER_DEF_
 #include "cairo-dock-icon-manager.h"
 
 // public (manager, config, data)
 CairoIconsParam myIconsParam;
-CairoIconsManager myIconsMgr;
+GldiManager myIconsMgr;
+GldiObjectManager myIconObjectMgr;
 CairoDockImageBuffer g_pIconBackgroundBuffer;
 GLuint g_pGradationTexture[2]={0, 0};
 
@@ -73,56 +79,10 @@ static void _cairo_dock_unload_icon_theme (void);
 static void _on_icon_theme_changed (GtkIconTheme *pIconTheme, gpointer data);
 
 
-void cairo_dock_free_icon (Icon *icon)
+void gldi_icons_foreach (GldiIconFunc pFunction, gpointer pUserData)
 {
-	if (icon == NULL)
-		return ;
-	cd_debug ("%s (%s , %s)", __func__, icon->cName, icon->cClass);
-	
-	cairo_dock_remove_dialog_if_any (icon);
-	if (icon->iSidRedrawSubdockContent != 0)
-		g_source_remove (icon->iSidRedrawSubdockContent);
-	if (icon->iSidLoadImage != 0)
-		g_source_remove (icon->iSidLoadImage);
-	if (icon->iSidDoubleClickDelay != 0)
-		g_source_remove (icon->iSidDoubleClickDelay);
-	if (CAIRO_DOCK_IS_NORMAL_APPLI (icon))
-		cairo_dock_unregister_appli (icon);
-	else if (icon->cClass != NULL)  // c'est un inhibiteur.
-		cairo_dock_deinhibite_class (icon->cClass, icon);
-	if (icon->pModuleInstance != NULL)
-		cairo_dock_deinstanciate_module (icon->pModuleInstance);
-	cairo_dock_notify_on_object (icon, NOTIFICATION_STOP_ICON, icon);
-	cairo_dock_notify_on_object (icon, NOTIFICATION_DESTROY, icon);
-	cairo_dock_remove_transition_on_icon (icon);
-	cairo_dock_remove_data_renderer_on_icon (icon);
-	
-	if (icon->iSpecificDesktop != 0)
-	{
-		s_iNbNonStickyLaunchers --;
-		s_pFloatingIconsList = g_list_remove(s_pFloatingIconsList, icon);
-	}
-	
-	cairo_dock_clear_notifications_on_object (icon);
-	
-	cairo_dock_free_icon_buffers (icon);
-	cd_debug ("icon freeed");
-	g_free (icon);
-}
-
-
-void cairo_dock_delete_icon_from_current_theme (Icon *icon)
-{
-	if (icon->iface.on_delete)
-	{
-		icon->iface.on_delete (icon);
-	}
-}
-
-void cairo_dock_foreach_icons (CairoDockForeachIconFunc pFunction, gpointer pUserData)
-{
-	cairo_dock_foreach_icons_in_docks (pFunction, pUserData);
-	cairo_dock_foreach_icons_in_desklets (pFunction, pUserData);
+	gldi_icons_foreach_in_docks (pFunction, pUserData);
+	gldi_desklets_foreach_icons (pFunction, pUserData);
 }
 
   /////////////////////////
@@ -136,7 +96,7 @@ void cairo_dock_foreach_icons (CairoDockForeachIconFunc pFunction, gpointer pUse
 static void _cairo_dock_detach_launcher (Icon *pIcon)
 {
 	gchar *cParentDockName = g_strdup (pIcon->cParentDockName);
-	cairo_dock_detach_icon_from_dock (pIcon, CAIRO_DOCK (pIcon->pContainer));  // this will set cParentDockName to NULL
+	gldi_icon_detach (pIcon);  // this will set cParentDockName to NULL
 	
 	pIcon->cParentDockName = cParentDockName;  // put it back !
 }
@@ -153,9 +113,9 @@ static void _hide_launcher_on_other_desktops (Icon *icon, int index)
 		}
 	}
 }
-static void _hide_icon_on_other_desktops (Icon *icon, G_GNUC_UNUSED CairoContainer *pContainer, gpointer data)
+static void _hide_icon_on_other_desktops (Icon *icon, gpointer data)
 {
-	if (CAIRO_DOCK_ICON_TYPE_IS_LAUNCHER (icon) || CAIRO_DOCK_ICON_TYPE_IS_CONTAINER (icon))
+	if (GLDI_OBJECT_IS_USER_ICON (icon))
 	{
 		int index = GPOINTER_TO_INT (data);
 		_hide_launcher_on_other_desktops (icon, index);
@@ -168,31 +128,31 @@ static void _show_launcher_on_this_desktop (Icon *icon, int index)
 		cd_debug (" => est visible sur ce viewport (iSpecificDesktop = %d).",icon->iSpecificDesktop);
 		s_pFloatingIconsList = g_list_remove (s_pFloatingIconsList, icon);
 		
-		CairoDock *pParentDock = cairo_dock_search_dock_from_name (icon->cParentDockName);
+		CairoDock *pParentDock = gldi_dock_get (icon->cParentDockName);
 		if (pParentDock != NULL)
 		{
-			cairo_dock_insert_icon_in_dock (icon, pParentDock, ! CAIRO_DOCK_ANIMATE_ICON);
+			gldi_icon_insert_in_container (icon, CAIRO_CONTAINER(pParentDock), ! CAIRO_DOCK_ANIMATE_ICON);
 		}
 		else  // the dock doesn't exist any more -> free the icon
 		{
 			icon->iSpecificDesktop = 0;  // pour ne pas qu'elle soit enlevee de la liste en parallele.
-			cairo_dock_free_icon (icon);
+			gldi_object_delete (GLDI_OBJECT (icon));
 		}
 	}
 }
 
-void cairo_dock_hide_show_launchers_on_other_desktops (void )
+void cairo_dock_hide_show_launchers_on_other_desktops (void )  /// TODO: add a mechanism to hide an icon in a dock (or even in a container ?) without detaching it...
 {
 	if (s_iNbNonStickyLaunchers <= 0)
 		return ;
 	
 	// calculate the index of the current desktop
 	int iCurrentDesktop = 0, iCurrentViewportX = 0, iCurrentViewportY = 0;
-	cairo_dock_get_current_desktop_and_viewport (&iCurrentDesktop, &iCurrentViewportX, &iCurrentViewportY);
+	gldi_desktop_get_current (&iCurrentDesktop, &iCurrentViewportX, &iCurrentViewportY);
 	int index = iCurrentDesktop * g_desktopGeometry.iNbViewportX * g_desktopGeometry.iNbViewportY + iCurrentViewportX * g_desktopGeometry.iNbViewportY + iCurrentViewportY + 1;  // +1 car on commence a compter a partir de 1.
 	
 	// first detach what shouldn't be shown on this desktop
-	cairo_dock_foreach_icons_in_docks ((CairoDockForeachIconFunc)_hide_icon_on_other_desktops, GINT_TO_POINTER (index));
+	gldi_icons_foreach_in_docks ((GldiIconFunc)_hide_icon_on_other_desktops, GINT_TO_POINTER (index));
 	
 	// then reattach what was eventually missing
 	Icon *icon;
@@ -209,7 +169,7 @@ void cairo_dock_hide_show_launchers_on_other_desktops (void )
 static gboolean _on_change_current_desktop_viewport_notification (G_GNUC_UNUSED gpointer data)
 {
 	cairo_dock_hide_show_launchers_on_other_desktops ();
-	return CAIRO_DOCK_LET_PASS_NOTIFICATION;
+	return GLDI_NOTIFICATION_LET_PASS;
 }
 
 static void _cairo_dock_delete_floating_icons (void)
@@ -220,7 +180,7 @@ static void _cairo_dock_delete_floating_icons (void)
 	{
 		icon = ic->data;
 		icon->iSpecificDesktop = 0;  // pour ne pas qu'elle soit enlevee de la liste en parallele.
-		cairo_dock_free_icon (icon);
+		gldi_object_unref (GLDI_OBJECT (icon));
 	}
 	g_list_free (s_pFloatingIconsList);
 	s_pFloatingIconsList = NULL;
@@ -339,7 +299,11 @@ gchar *cairo_dock_search_icon_s_path (const gchar *cFileName, gint iDesiredIconS
 		{
 			g_string_assign (sIconPath, gtk_icon_info_get_filename (pIconInfo));
 			bFileFound = TRUE;
+			#if GTK_CHECK_VERSION (3, 8, 0)
+			g_object_unref (G_OBJECT (pIconInfo));
+			#else
 			gtk_icon_info_free (pIconInfo);
+			#endif
 		}
 	}
 	
@@ -739,13 +703,13 @@ static void _reload_in_desklet (CairoDesklet *pDesklet, G_GNUC_UNUSED gpointer d
 {
 	if (CAIRO_DOCK_IS_APPLET (pDesklet->pIcon))
 	{
-		cairo_dock_reload_module_instance (pDesklet->pIcon->pModuleInstance, FALSE);
+		gldi_object_reload (GLDI_OBJECT(pDesklet->pIcon->pModuleInstance), FALSE);
 	}
 }
 static gboolean _on_icon_theme_changed_idle (G_GNUC_UNUSED gpointer data)
 {
 	cd_debug ("");
-	cairo_dock_foreach_desklet ((CairoDockForeachDeskletFunc) _reload_in_desklet, NULL);
+	gldi_desklets_foreach ((GldiDeskletForeachFunc) _reload_in_desklet, NULL);
 	cairo_dock_reload_buffers_in_all_docks (FALSE);
 	s_iSidReloadTheme = 0;
 	return FALSE;
@@ -791,53 +755,69 @@ static void load (void)
  /// RELOAD ///
 //////////////
 
-static void _remove_separators (G_GNUC_UNUSED const gchar *cDockName, CairoDock *pDock, G_GNUC_UNUSED gpointer data)
-{
-	cairo_dock_remove_automatic_separators (pDock);
-}
-static void _insert_separators (G_GNUC_UNUSED const gchar *cDockName, CairoDock *pDock, G_GNUC_UNUSED gpointer data)
+static void _reload_separators (G_GNUC_UNUSED const gchar *cDockName, CairoDock *pDock, gpointer data)
 {
+	///cairo_dock_remove_automatic_separators (pDock);
+	gboolean bSeparatorsNeedReload = GPOINTER_TO_INT(data);
+	gboolean bHasSeparator = FALSE;
 	Icon *icon;
 	GList *ic;
-	for (ic = pDock->icons; ic != NULL; ic = ic->next)  // les separateurs utilisateurs ne sont pas recrees, on les recharge donc.
+	for (ic = pDock->icons; ic != NULL; ic = ic->next)
 	{
 		icon = ic->data;
-		if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))  // il n'y a que des separateurs utilisateurs dans le dock en ce moment.
+		if (GLDI_OBJECT_IS_SEPARATOR_ICON (icon))
 		{
-			cairo_dock_load_icon_image (icon, CAIRO_CONTAINER (pDock));
+			if (bSeparatorsNeedReload)
+			{
+				cairo_dock_icon_set_requested_size (icon, 0, 0);
+				cairo_dock_set_icon_size_in_dock (pDock, icon);
+			}
+			cairo_dock_load_icon_image (icon, icon->pContainer);
+			bHasSeparator = TRUE;
 		}
 	}
-	cairo_dock_insert_automatic_separators_in_dock (pDock);
+	if (bHasSeparator)
+	{
+		if (bSeparatorsNeedReload)
+			cairo_dock_update_dock_size (pDock);  // either to trigger the loading of the separator rendering, or to take into account the change in the separators size
+		gtk_widget_queue_draw (pDock->container.pWidget);  // in any case, refresh the drawing
+	}
 }
+
 static void _calculate_icons (G_GNUC_UNUSED const gchar *cDockName, CairoDock *pDock, G_GNUC_UNUSED gpointer data)
 {
 	cairo_dock_calculate_dock_icons (pDock);
 }
 
-static void _reload_one_label (Icon *pIcon, G_GNUC_UNUSED CairoContainer *pContainer, G_GNUC_UNUSED CairoIconsParam *pLabels)
+static void _reload_one_label (Icon *pIcon, G_GNUC_UNUSED gpointer data)
 {
 	cairo_dock_load_icon_text (pIcon);
 	cairo_dock_load_icon_quickinfo (pIcon);
 }
-static void _cairo_dock_resize_one_dock (G_GNUC_UNUSED gchar *cDockName, CairoDock *pDock, G_GNUC_UNUSED gpointer data)
+static void _cairo_dock_resize_one_dock (G_GNUC_UNUSED const gchar *cDockName, CairoDock *pDock, G_GNUC_UNUSED gpointer data)
 {
 	cairo_dock_update_dock_size (pDock);
 }
 
 static void reload (CairoIconsParam *pPrevIcons, CairoIconsParam *pIcons)
 {
-	gboolean bInsertSeparators = FALSE;
+	// if the separator size has changed, we need to re-allocate it, reload the image, and update the dock size
+	// if the separator rendering has changed (type or color), we need to load the new rendering, which is done by the View (during the compute_size)
+	// otherwise, we just need to redraw 
+	gboolean bSeparatorsNeedReload = (pPrevIcons->iSeparatorWidth != pIcons->iSeparatorWidth
+		|| pPrevIcons->iSeparatorHeight != pIcons->iSeparatorHeight
+		|| pPrevIcons->iSeparatorType != pIcons->iSeparatorType
+		|| cairo_dock_colors_differ (pPrevIcons->fSeparatorColor, pIcons->fSeparatorColor));  // same if color has changed (for the flat separator rendering)
+	gboolean bSeparatorNeedRedraw = (g_strcmp0 (pPrevIcons->cSeparatorImage, pIcons->cSeparatorImage) != 0
+		|| pPrevIcons->bRevolveSeparator != pIcons->bRevolveSeparator);
 	
-	if (cairo_dock_strings_differ (pPrevIcons->cSeparatorImage, pIcons->cSeparatorImage) ||
-		pPrevIcons->iSeparatorWidth != pIcons->iSeparatorWidth ||
-		pPrevIcons->iSeparatorHeight != pIcons->iSeparatorHeight ||
-		pPrevIcons->fAmplitude != pIcons->fAmplitude)
+	if (bSeparatorsNeedReload || bSeparatorNeedRedraw)
 	{
-		bInsertSeparators = TRUE;
-		cairo_dock_foreach_docks ((GHFunc)_remove_separators, NULL);
+		gldi_docks_foreach ((GHFunc)_reload_separators, GINT_TO_POINTER(bSeparatorsNeedReload));
+		return;
 	}
 	
-	gboolean bThemeChanged = cairo_dock_strings_differ (pIcons->cIconTheme, pPrevIcons->cIconTheme);
+	gboolean bThemeChanged = (g_strcmp0 (pIcons->cIconTheme, pPrevIcons->cIconTheme) != 0);
 	if (bThemeChanged)
 	{
 		_cairo_dock_unload_icon_theme ();
@@ -847,8 +827,8 @@ static void reload (CairoIconsParam *pPrevIcons, CairoIconsParam *pIcons)
 	
 	gboolean bIconBackgroundImagesChanged = FALSE;
 	// if background images are different, reload them and trigger the reload of all icons
-	if (cairo_dock_strings_differ (pPrevIcons->cBackgroundImagePath, pIcons->cBackgroundImagePath) ||
-		pPrevIcons->fAmplitude != pIcons->fAmplitude)
+	if (g_strcmp0 (pPrevIcons->cBackgroundImagePath, pIcons->cBackgroundImagePath) != 0
+	|| pPrevIcons->fAmplitude != pIcons->fAmplitude)
 	{
 		bIconBackgroundImagesChanged = TRUE;
 		_cairo_dock_load_icons_background_surface (pIcons->cBackgroundImagePath);
@@ -871,33 +851,28 @@ static void reload (CairoIconsParam *pPrevIcons, CairoIconsParam *pIcons)
 		cairo_dock_reload_buffers_in_all_docks (TRUE);
 	}
 	
-	if (bInsertSeparators)
-	{
-		cairo_dock_foreach_docks ((GHFunc)_insert_separators, NULL);
-	}
-	
 	if (pPrevIcons->iIconWidth != pIcons->iIconWidth ||
 		pPrevIcons->iIconHeight != pIcons->iIconHeight ||
 		pPrevIcons->fAmplitude != pIcons->fAmplitude)
 	{
 		_cairo_dock_unload_icon_textures ();
-		myIndicatorsMgr.mgr.unload ();
+		myIndicatorsMgr.unload ();
 		_cairo_dock_load_icon_textures ();
-		myIndicatorsMgr.mgr.load ();
+		myIndicatorsMgr.load ();
 	}
 	
 	cairo_dock_set_all_views_to_default (0);  // met a jour la taille (decorations incluses) de tous les docks; le chargement des separateurs plats se fait dans le calcul de max dock size.
-	cairo_dock_foreach_docks ((GHFunc)_calculate_icons, NULL);
-	cairo_dock_redraw_root_docks (FALSE);  // main dock inclus.
+	gldi_docks_foreach ((GHFunc)_calculate_icons, NULL);
+	gldi_docks_redraw_all_root ();
 	
 	// labels
 	CairoIconsParam *pLabels = pIcons;
 	CairoIconsParam *pPrevLabels = pPrevIcons;
-	cairo_dock_foreach_icons ((CairoDockForeachIconFunc) _reload_one_label, pLabels);
+	gldi_icons_foreach ((GldiIconFunc) _reload_one_label, NULL);
 	
 	if (pPrevLabels->iLabelSize != pLabels->iLabelSize)
 	{
-		cairo_dock_foreach_docks ((GHFunc) _cairo_dock_resize_one_dock, NULL);
+		gldi_docks_foreach ((GHFunc) _cairo_dock_resize_one_dock, NULL);
 	}
 }
 
@@ -956,43 +931,131 @@ static void unload (void)
 
 static void init (void)
 {
-	cairo_dock_register_notification_on_object (&myDesktopMgr,
+	gldi_object_register_notification (&myDesktopMgr,
 		NOTIFICATION_DESKTOP_CHANGED,
-		(CairoDockNotificationFunc) _on_change_current_desktop_viewport_notification,
-		CAIRO_DOCK_RUN_AFTER, NULL);
-	cairo_dock_register_notification_on_object (&myIconsMgr,
+		(GldiNotificationFunc) _on_change_current_desktop_viewport_notification,
+		GLDI_RUN_AFTER, NULL);
+	gldi_object_register_notification (&myIconObjectMgr,
 		NOTIFICATION_RENDER_ICON,
-		(CairoDockNotificationFunc) cairo_dock_render_icon_notification,
-		CAIRO_DOCK_RUN_FIRST, NULL);
+		(GldiNotificationFunc) cairo_dock_render_icon_notification,
+		GLDI_RUN_FIRST, NULL);
 	
 }
 
-
   ///////////////
  /// MANAGER ///
 ///////////////
 
+static void _load_image (Icon *icon)
+{
+	int iWidth = cairo_dock_icon_get_allocated_width (icon);
+	int iHeight = cairo_dock_icon_get_allocated_height (icon);
+	cairo_surface_t *pSurface = NULL;
+	
+	if (icon->cFileName)
+	{
+		gchar *cIconPath = cairo_dock_search_icon_s_path (icon->cFileName, MAX (iWidth, iHeight));
+		if (cIconPath != NULL && *cIconPath != '\0')
+			pSurface = cairo_dock_create_surface_from_image_simple (cIconPath,
+				iWidth,
+				iHeight);
+		g_free (cIconPath);
+	}
+	cairo_dock_load_image_buffer_from_surface (&icon->image, pSurface, iWidth, iHeight);
+}
+static void init_object (GldiObject *obj, G_GNUC_UNUSED gpointer attr)
+{
+	Icon *icon = (Icon*)obj;
+	icon->iface.load_image = _load_image;
+}
+
+static void reset_object (GldiObject *obj)
+{
+	Icon *icon = (Icon*)obj;
+	cd_debug ("%s (%s , %s, %s)", __func__, icon->cName, icon->cClass, gldi_object_get_type(icon));
+	
+	GldiContainer *pContainer = cairo_dock_get_icon_container (icon);
+	if (pContainer != NULL)
+	{
+		gldi_icon_detach (icon);
+	}
+	
+	if (icon->cClass != NULL && (GLDI_OBJECT_IS_LAUNCHER_ICON (icon) || GLDI_OBJECT_IS_APPLET_ICON (icon)))  // c'est un inhibiteur.
+		cairo_dock_deinhibite_class (icon->cClass, icon);  // unset the appli if it had any
+	
+	gldi_object_notify (icon, NOTIFICATION_STOP_ICON, icon);
+	cairo_dock_remove_transition_on_icon (icon);
+	cairo_dock_remove_data_renderer_on_icon (icon);
+	
+	if (icon->pSubDock != NULL)
+		gldi_object_unref (GLDI_OBJECT(icon->pSubDock));
+	
+	if (icon->iSpecificDesktop != 0)
+	{
+		s_iNbNonStickyLaunchers --;
+		s_pFloatingIconsList = g_list_remove(s_pFloatingIconsList, icon);
+	}
+	
+	if (icon->iSidRedrawSubdockContent != 0)
+		g_source_remove (icon->iSidRedrawSubdockContent);
+	if (icon->iSidLoadImage != 0)  // remove timers after any function that could trigger one (for instance, cairo_dock_deinhibite_class calls cairo_dock_trigger_load_icon_buffers)
+		g_source_remove (icon->iSidLoadImage);
+	if (icon->iSidDoubleClickDelay != 0)
+		g_source_remove (icon->iSidDoubleClickDelay);
+	
+	// free data
+	g_free (icon->cDesktopFileName);
+	g_free (icon->cFileName);
+	g_free (icon->cName);
+	g_free (icon->cInitialName);
+	g_free (icon->cCommand);
+	g_free (icon->cWorkingDirectory);
+	g_free (icon->cBaseURI);
+	g_free (icon->cParentDockName);  // on ne liberera pas le sous-dock ici sous peine de se mordre la queue, donc il faut l'avoir fait avant.
+	g_free (icon->cClass);
+	g_free (icon->cWmClass);
+	g_free (icon->cQuickInfo);
+	///g_free (icon->cLastAttentionDemand);
+	g_free (icon->pHiddenBgColor);
+	if (icon->pMimeTypes)
+		g_strfreev (icon->pMimeTypes);
+	
+	cairo_dock_unload_image_buffer (&icon->image);
+	
+	cairo_dock_unload_image_buffer (&icon->label);
+	
+	cairo_dock_destroy_icon_overlays (icon);
+}
+
 void gldi_register_icons_manager (void)
 {
 	// Manager
-	memset (&myIconsMgr, 0, sizeof (CairoIconsManager));
-	myIconsMgr.mgr.cModuleName 	= "Icons";
-	myIconsMgr.mgr.init 		= init;
-	myIconsMgr.mgr.load 		= load;
-	myIconsMgr.mgr.unload 		= unload;
-	myIconsMgr.mgr.reload 		= (GldiManagerReloadFunc)reload;
-	myIconsMgr.mgr.get_config 	= (GldiManagerGetConfigFunc)get_config;
-	myIconsMgr.mgr.reset_config = (GldiManagerResetConfigFunc)reset_config;
+	memset (&myIconsMgr, 0, sizeof (GldiManager));
+	gldi_object_init (GLDI_OBJECT(&myIconsMgr), &myManagerObjectMgr, NULL);
+	myIconsMgr.cModuleName  = "Icons";
+	// interface
+	myIconsMgr.init         = init;
+	myIconsMgr.load         = load;
+	myIconsMgr.unload       = unload;
+	myIconsMgr.reload       = (GldiManagerReloadFunc)reload;
+	myIconsMgr.get_config   = (GldiManagerGetConfigFunc)get_config;
+	myIconsMgr.reset_config = (GldiManagerResetConfigFunc)reset_config;
 	// Config
 	memset (&myIconsParam, 0, sizeof (CairoIconsParam));
-	myIconsMgr.mgr.pConfig = (GldiManagerConfigPtr)&myIconsParam;
-	myIconsMgr.mgr.iSizeOfConfig = sizeof (CairoIconsParam);
+	myIconsMgr.pConfig = (GldiManagerConfigPtr)&myIconsParam;
+	myIconsMgr.iSizeOfConfig = sizeof (CairoIconsParam);
 	// data
 	memset (&g_pIconBackgroundBuffer, 0, sizeof (CairoDockImageBuffer));
-	myIconsMgr.mgr.pData = (GldiManagerDataPtr)NULL;
-	myIconsMgr.mgr.iSizeOfData = 0;
+	myIconsMgr.pData = (GldiManagerDataPtr)NULL;
+	myIconsMgr.iSizeOfData = 0;
+	
+	// ObjectManager
+	memset (&myIconObjectMgr, 0, sizeof (GldiObjectManager));
+	myIconObjectMgr.cName        = "Icon";
+	myIconObjectMgr.iObjectSize  = sizeof (Icon);
+	// interface
+	myIconObjectMgr.init_object  = init_object;
+	myIconObjectMgr.reset_object = reset_object;
 	// signals
-	cairo_dock_install_notifications_on_object (&myIconsMgr, NB_NOTIFICATIONS_ICON);
-	// register
-	gldi_register_manager (GLDI_MANAGER(&myIconsMgr));
+	gldi_object_install_notifications (&myIconObjectMgr, NB_NOTIFICATIONS_ICON);
 }
